# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-30 13:06
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('testapp', '0002_tag'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='publisher',
            options={'permissions': (('can_publish', 'Can publish'),)},
        ),
    ]
