# -*- coding: utf-8 -*-

from chemtrails.settings import chemtrails_settings as settings

default_app_config = 'chemtrails.apps.ChemtrailsConfig'

__title__ = 'django-chemtrails'
__version__ = '0.0.2'
__author__ = 'Rolf Haavard Blindheim'
__license__ = 'MIT License'

VERSION = __version__

__all__ = [
    'settings'
]
