# -*- coding: utf-8 -*-

from django.test import TestCase


class ArrayChoiceFieldTestCase(TestCase):

    def test_display(self):
        pass

    def test_to_python(self):
        pass

    def test_validate(self):
        pass
